import dork_compose.plugin


class Plugin(dork_compose.plugin.Plugin):

    def preprocess_config(self, config):
        config.services.append({
            'name': 'dork_tracker',
            'image': 'alpine:3.4',
            'command': 'tail -f /dev/null',
            'volumes': [],
            'labels': {
                'dork.tracker': '',
                'dork.tracker.source': self.env.get('DORK_SOURCE'),
                'dork.tracker.project': self.env.get('DORK_PROJECT'),
                'dork.tracker.instance': self.env.get('DORK_INSTANCE'),
            }
        })




import dork_compose.plugin
import os

from compose.config.environment import env_vars_from_file

class Plugin(dork_compose.plugin.Plugin):
    def environment(self):
        old_environment = os.environ.copy()
        path = filter(len, os.path.abspath(os.path.curdir).split('/'))
        current = ''
        while len(path):
            current = current + '/' + path.pop(0)
            envfile = '%s/.dork.env' % current
            if os.path.isfile(envfile):
                for key, value in env_vars_from_file(envfile).iteritems():
                    os.environ[key] = os.path.expandvars(value)
                continue

            envfile = '%s/.env' % current
            if os.path.isfile(envfile):
                for key, value in env_vars_from_file(envfile).iteritems():
                    os.environ[key] = os.path.expandvars(value)
        env = os.environ.copy()
        os.environ = old_environment
        return env





import os


def tru(val):
    return val


def notdefault(val):
    return val if val != 'default' else None


def is_subdir(path, directory):
    path = os.path.realpath(path)
    directory = os.path.realpath(directory)
    relative = os.path.relpath(path, directory)
    return not (relative == os.pardir or relative.startswith(os.pardir + os.sep))



import dork_compose.plugin
import os
import glob
import dork_compose.helpers

from compose.config.environment import env_vars_from_file


class Plugin(dork_compose.plugin.Plugin):

    @property
    def library(self):
        return '/'.join(filter(lambda x: len(x), [
            os.path.expanduser(self.env.get('DORK_LIBRARY_PATH', '')),
            os.path.expanduser(self.env.get('DORK_LIBRARY', '')),
        ]))

    def environment(self):
        env = {}
        layout = self.env.get('DORK_DIRECTORY_LAYOUT', False)
        source = self.env.get('DORK_SOURCE')
        if layout:
            matches = [path for path in glob.glob(os.path.abspath(layout)) if dork_compose.helpers.is_subdir(source, path)]
            if not matches:
                return env
            env['DORK_SOURCE'] = max(matches, key=len)
            if env['DORK_SOURCE'] != source:
                hotcode = [path for path in env.get('DORK_HOTCODE', '').split(';') if path != '']
                hotcode.append(source[len(env['DORK_SOURCE'])+1:])
                env['DORK_HOTCODE'] = ';'.join(hotcode)

        if self.library:
            files = filter(lambda x: x, self.env.get('COMPOSE_FILE', '').split(':'))
            if os.path.isfile(self.library + '/docker-compose.yml'):
                files.insert(0, self.library + '/docker-compose.yml')

            os.environ.update({
                'COMPOSE_FILE': ':'.join(files)
            })

            envfile = '%s/.env' % self.library
            if os.path.isfile(envfile):
                for key, value in env_vars_from_file(envfile).iteritems():
                    if key not in self.env:
                        env[key] = os.path.expandvars(value)
        return env






import dork_compose.plugin
import os
import shutil
from dork_compose.helpers import is_subdir

import six
import sys
from compose.progress_stream import stream_output, StreamOutputError
from compose.service import BuildError, NoSuchImageError
from docker.errors import NotFound


class Plugin(dork_compose.plugin.Plugin):

    def __init__(self, env, name, command):
        super(Plugin, self).__init__(env, name, command)
        self.clean_paths = []

    def alter_config_schema(self, schema):
        schema['definitions']['service']['properties']['build']['oneOf'][1]['properties'].update({
            'source': {'type':'string'},
            'onbuild': {'type':'string'},
        })

        schema['definitions']['constraints']['service']['properties']['build'] = {
            'anyOf': [
                {'required': ['context']},
                {'required': ['onbuild']},
            ]
        }

    def building(self, service, no_cache, pull, force_rm):
        

        context = service.options.get('build', {}).get('context', None)
        source = service.options.get('build', {}).get('source', '.')
        onbuild = service.options.get('build', {}).get('onbuild', None)

        if not onbuild and context and not is_subdir(context, self.env['DORK_SOURCE']):
            dockerfile = service.options.get('build', {}).get('dockerfile', None)

            onbuild = "%s/%s:autobuild" % (
                os.path.basename(self.env.get('DORK_LIBRARY', self.project)),
                service.name
            )

            build_output = service.client.build(
                path=context,
                tag=onbuild,
                pull=pull,
                forcerm=force_rm,
                nocache=no_cache,
                dockerfile=dockerfile,
            )
            try:
                stream_output(build_output, sys.stdout)
            except StreamOutputError as e:
                raise BuildError(self, six.text_type(e))

        if onbuild:
            self.clean_paths.append(source)
            dockerfile = '%s/Dockerfile' % source
            dockerignore = '%s/.dockerignore' % source

            if os.path.isfile(dockerfile):
                os.rename(dockerfile, '%s/.dork.Dockerfile' % source)

            if os.path.isfile(dockerignore):
                shutil.copyfile(dockerignore, '%s/.dork.dockerignore' % source)
            else:
                open('%s/.dockerignore' % source, 'a').close()

            with open(dockerfile, 'w') as f:
                # TODO: don't rely on virtualport
                # proxy plugin should not modify config so we can use real ports
                virtual_port = service.options.get('environment', {}).get('VIRTUAL_PORT', None)
                if virtual_port:
                    f.write('FROM %s \nLABEL dork.source="%s"\nEXPOSE %s' % (onbuild, source, virtual_port))
                else:
                    f.write('FROM %s \nLABEL dork.source="%s"' % (onbuild, source))

            try:
                image = service.client.inspect_image(onbuild)
            except NotFound:
                service.client.pull(onbuild)
                image = service.client.inspect_image(onbuild)

            ignore = (filter(lambda x: x, image.get('Config', {})
                                   .get('Labels', {})
                                   .get('dork.hotcode', '')
                                   .split(';')))

            if isinstance(service.options.get('labels'), dict) and 'dork.hotcode' in service.options['labels']:
                ignore = [path for path in service.options.get('labels').get('dork.hotcode', '').split(';') if path != '']

            ignore.append('.dockerignore')
            ignore.append('Dockerfile')
            ignore.append('.dork.dockerignore')
            ignore.append('.dork.Dockerfile')
            ignore.append('.git')

            with open(dockerignore, 'a') as f:
                f.write('\n' + '\n'.join(ignore))

            service.options['build']['context'] = os.path.abspath(source)

    def cleanup(self):
        for path in self.clean_paths:
            if os.path.isfile('%s/.dork.Dockerfile' % path):
                os.remove('%s/Dockerfile' % path)
                os.rename('%s/.dork.Dockerfile' % path, '%s/Dockerfile' % path)
            elif os.path.isfile('%s/Dockerfile' % path):
                os.remove('%s/Dockerfile' % path)
            if os.path.isfile('%s/.dork.dockerignore' % path):
                os.remove('%s/.dockerignore' % path)
                os.rename('%s/.dork.dockerignore' % path, '%s/.dockerignore' % path)
            elif os.path.isfile('%s/.dockerignore' % path):
                os.remove('%s/.dockerignore' % path)



import dork_compose.plugin
from git import Repo
from gitdb.exc import BadName


class Plugin(dork_compose.plugin.Plugin):

    def __init__(self, env, name, command):
        dork_compose.plugin.Plugin.__init__(self, env, name, command)
        try:
            self.__repo = Repo(self.basedir)
        except Exception:
            self.__repo = False

    def __commit(self, hash):
        if not self.__repo:
            return None
        try:
            return self.__repo.commit(hash)
        except BadName:
            return None

    def snapshot_autosave(self):
        if not self.__repo:
            return None
        return self.__repo.head.commit.hexsha

    def snapshot_autoload(self, snapshots=()):
        if not self.__repo:
            return None
        name = None
        head = self.__repo.head.commit
        distance = -1
        for commit in filter(lambda x: x, map(self.__commit, snapshots)):
            if self.__repo.is_ancestor(commit.hexsha, head.hexsha):
                diff = len(commit.diff(head))
                if diff < distance or distance == -1:
                    name = commit.hexsha
                    distance = diff
        return name

    def snapshot_autoclean(self, snapshots=()):
        if not self.__repo:
            return None
        snapshots = filter(lambda x: x, map(self.__commit, snapshots))
        names = []
        for current in snapshots:
            for test in snapshots:
                if self.__repo.is_ancestor(test, current) and test != current and test.hexsha not in names:
                    names.append(test.hexsha)
        return names




import dork_compose.plugin
import platform
import pkg_resources
from subprocess import call
import tempfile
import os

import logging
log = logging.getLogger(__name__)

class Plugin(dork_compose.plugin.Plugin):

    def environment(self):
        return {
            'DORK_DNS_PORT': self.dns_port,
            'DORK_DNS_HOST': self.dns_host,
            'DORK_PROXY_DOMAIN': self.proxy_domain,
        }

    @property
    def auxiliary_project(self):
        return pkg_resources.resource_filename('dork_compose', 'auxiliary/dns')

    @property
    def proxy_domain(self):
        return self.env.get('DORK_PROXY_DOMAIN', 'dork.io')

    @property
    def dns_port(self):
        return self.env.get('DORK_DNS_PORT', '53')

    @property
    def dns_host(self):
        return self.env.get('DORK_DNS_HOST', '127.0.0.1')

    def initializing(self, project, service_names=None):
        if platform.system() == "Darwin":
            resolver = '/etc/resolver/%s' % self.env.get('DORK_PROXY_DOMAIN')
            log.warn("Root password required to update DNS settings.")
            if not os.path.isfile(resolver):
                content = "# Generated by dork-composer\nnameserver %s\nport %s" % (
                    self.environment()['DORK_DNS_HOST'],
                    self.environment()['DORK_DNS_PORT']
                )
                tmp = tempfile.NamedTemporaryFile(delete=False)
                tmp.write(content)
                tmp.close()
                call(['sudo', 'mkdir', '-p', '/etc/resolver'])
                call(['sudo', 'cp', tmp.name, resolver])
                call(['sudo', 'chmod', '664', resolver])
                os.remove(tmp.name)
            call(['sudo', 'dscacheutil', '-flushcache'])
            call(['sudo', 'killall', '-HUP', 'mDNSResponder'])

        if platform.system() == "Linux":
            resolver = '/etc/resolv.conf'
            confline = 'nameserver %s' % self.environment()['DORK_DNS_HOST']

            lines = []
            lines.append(confline + '\n')
            with open(resolver, 'r') as resolvconf:
                for line in resolvconf.readlines():
                    if confline in line:
                        return
                    lines.append(line)

            lines.append('\n')
            tmp = tempfile.NamedTemporaryFile(delete=False)
            tmp.write(''.join(lines))
            tmp.close()
            log.warn("Root password required to update DNS settings.")
            call(['sudo', 'cp', tmp.name, resolver])
            call(['sudo', 'chmod', '664', resolver])
            os.remove(tmp.name)



import dork_compose.plugin
from compose.cli.docker_client import docker_client
from dork_compose.helpers import notdefault, tru
import os
import urlparse
import pkg_resources
from subprocess import check_call

import logging
log = logging.getLogger(__name__)


class Plugin(dork_compose.plugin.Plugin):

    def __init__(self, env, name, command):
        dork_compose.plugin.Plugin.__init__(self, env, name, command)

    def environment(self):
        return {
            'DORK_PROXY_HTTPS_METHOD': self.https_method,
            'DORK_PROXY_HTTPS_SIGNING': self.https_signing,
            'DOCKER_SOCK': self.docker_sock,
            'DORK_PROXY_AUTH_DIR': self.auth_dir,
            'DORK_PROXY_CERTS_DIR': self.certs_dir,
            'DORK_PROXY_DOMAIN': self.proxy_domain,
            'DORK_PROXY_INSTANCE_DOMAIN': self.service_domain(),
            'DORK_PROXY_LETSENCRYPT_EMAIL': self.letsencrypt_email,
        }

    @property
    def https_signing(self):
        return self.env.get('DORK_PROXY_HTTPS_SIGNING', 'selfsigned')

    @property
    def auxiliary_project(self):
        return pkg_resources.resource_filename('dork_compose', 'auxiliary/proxy/%s' % self.https_signing)

    def service_domain(self, service=None):
        return '--'.join(filter(tru, [
            service,
            notdefault(self.project),
            notdefault(self.instance)
        ])) + '.' + self.proxy_domain

    def info(self, project):
        info = {}

        auth = self.collect_auth_files([service.name for service in project.services])
        for service in project.services:
            if 'environment' in service.options and 'VIRTUAL_HOST' in service.options['environment']:
                key = '%s url' % service.name
                info[key] = service.options['environment'].get('VIRTUAL_PROTO', 'http') + '://' + service.options['environment']['VIRTUAL_HOST']
                if service.name in auth and auth[service.name]:
                    info[key] += ' (password protected)'
        return info

    @property
    def https_method(self):
        return os.path.expanduser(self.env.get('DORK_PROXY_HTTPS_METHOD', 'noredirect'))

    @property
    def auth_dir(self):
        return os.path.expanduser(self.env.get('DORK_PROXY_AUTH_DIR', '%s/auth' % self.datadir))

    @property
    def certs_dir(self):
        return os.path.expanduser(self.env.get('DORK_PROXY_CERTS_DIR', '%s/certs' % self.datadir))

    @property
    def docker_sock(self):
        result = urlparse.urlparse(self.env.get('DOCKER_HOST', 'unix:///var/run/docker.sock'))
        if result.scheme != 'unix':
            raise EnvironmentError('Dork proxy works with docker socket api only.')
        return result.path

    @property
    def proxy_domain(self):
        return self.env.get('DORK_PROXY_DOMAIN', 'dork.io')

    @property
    def letsencrypt_email(self):
        return self.env.get('DORK_PROXY_LETSENCRYPT_EMAIL', 'admin@localhost')

    def reload_proxy(self):
        client = docker_client(self.env)
        containers = client.containers(all=True, filters={
            'label': 'org.iamdork.proxy'
        })

        for container in containers:
            ex = client.exec_create(container, 'nginx -s reload')
            client.exec_start(ex)

    def preprocess_config(self, config):

        for service in config.services:
            if 'ports' in service:
                indices = range(len(service['ports']))
                indices.reverse()
                for index in indices:
                    port = service['ports'][index]
                    if isinstance(port, basestring):
                        if ':' not in port:
                            continue
                        (external, internal) = port.split(':')
                        if external and internal:
                            domain = self.service_domain() if external == '80' or external == '443' else self.service_domain(service['name'])
                            if 'environment' not in service:
                                service['environment'] = {}
                            service['environment']['VIRTUAL_HOST'] = domain
                            service['environment']['LETSENCRYPT_HOST'] = domain
                            service['environment']['LETSENCRYPT_EMAIL'] = self.letsencrypt_email
                            if 'labels' not in service:
                                service['labels'] = {}
                            service['environment']['VIRTUAL_PORT'] = int(internal)
                            service['ports'][index] = internal

    def collect_auth_files(self, services):
        files = {service: [] for service in services}

        path = filter(len, self.basedir.split('/'))
        current = ''
        while len(path):
            current = current + '/' + path.pop(0)

            auth = '%s/.auth' % current
            if os.path.isfile(auth):
                with open(auth) as f:
                    auth_data = f.read()
                    for service in services:
                        files[service].append(auth_data)

            no_auth = '%s/.no_auth' % current
            if os.path.isfile(no_auth):
                for service in services:
                    if service in files:
                        files[service] = []

            for service in services:
                auth = '%s/.auth.%s' % (current, service)
                no_auth = '%s/.no_auth.%s' % (current, service)
                if os.path.isfile(auth):
                    with open(auth) as f:
                        files[service].append(f.read())
                if os.path.isfile(no_auth):
                    files[service] = []

        return files

    def initializing(self, project, service_names=None):
        if self.https_signing == 'selfsigned':
            if not os.path.isdir(self.certs_dir):
                os.makedirs(self.certs_dir)

            if not os.path.isfile(self.certs_dir + '/dhparam.pem'):
                log.info("Creating Diffie-Hellman group. This might take a while.")
                check_call(['openssl', 'dhparam', '-out', '%s/dhparam.pem' % self.certs_dir, '2048'])

            key = '%s/%s.key' % (self.certs_dir, self.proxy_domain)
            crt = '%s/%s.crt' % (self.certs_dir, self.proxy_domain)

            if not os.path.isfile(key) or not os.path.isfile(crt):
                log.info("Creating self signed key and certificate for domain '%s'." % self.proxy_domain)
                check_call([
                    'openssl', 'req', '-x509', '-nodes',
                    '-days', '365', '-newkey', 'rsa:2048',
                    '-keyout', key,
                    '-out', crt,
                    # TODO: fix signed parameters
                    '-subj', '/C=GB/ST=London/L=London/O=Global Security/OU=IT Department/CN=*.%s' % self.proxy_domain
                ])

        auth = self.collect_auth_files([service.name for service in project.get_services()])

        for service in project.get_services():
            if self.auth_dir and 'environment' in service.options and 'VIRTUAL_HOST' in service.options['environment']:
                lines = auth[service.name]
                authfile = '%s/%s' % (self.auth_dir, service.options['environment']['VIRTUAL_HOST'])
                if lines:
                    if not os.path.isdir(self.auth_dir):
                        os.makedirs(self.auth_dir)
                    with open(authfile, mode='w+') as f:
                        f.writelines(lines)
                elif os.path.exists(authfile):
                    os.remove(authfile)

    def removed(self, project, include_volumes=False):
        for service in project.get_services():
            if self.auth_dir and 'environment' in service.options and 'VIRTUAL_HOST' in service.options['environment']:
                authfile = '%s/%s' % (self.auth_dir, service.options['environment']['VIRTUAL_HOST'])
                if os.path.isfile(authfile):
                    os.remove(authfile)





import os
import contextlib

from compose.cli.command import get_client
from compose.cli.docker_client import docker_client
from compose.config import config
from compose.config.environment import Environment
from dork_compose.injections import dork_config_load
from compose.const import API_VERSIONS
from compose.project import Project
from helpers import notdefault, tru
import pkg_resources
import filelock

import logging
log = logging.getLogger(__name__)


@contextlib.contextmanager
def load(plugins, command):
    instances = []

    environment = {
        'DORK_PROJECT': 'default',
        'DORK_INSTANCE': 'default',
        'DORK_SOURCE': os.path.abspath(os.curdir),
        'DORK_DATA_DIR': '~/.dork',
    }
    environment.update(os.environ)

    for plugin in plugins.split(':'):
        local = {}
        if '=' in plugin:
            (plugin, f) = plugin.split('=')
        else:
            f = "%s/%s.py" % (pkg_resources.resource_filename('dork_compose', 'plugins'), plugin)

        f = os.path.expanduser(f)

        try:
            execfile(os.path.expanduser(f), local)
            instances.append(local['Plugin'](environment.copy(), plugin, command))
            environment.update(instances[-1].environment())

            log.debug('Loaded plugin %s.' % plugin)
        except Exception as ex:
            log.warning('Could not load plugin %s: %s' % (plugin, ex))
            pass

    # If there is no explicit project name in the environment, set
    # [project]--[instance]
    if 'COMPOSE_PROJECT_NAME' not in environment:
        parts = filter(tru, [
            notdefault(environment['DORK_PROJECT']),
            notdefault(environment['DORK_INSTANCE'])
        ])
        if parts:
            environment.update({
                'COMPOSE_PROJECT_NAME': '--'.join(parts)
            })

    os.environ.update(environment)

    try:
        yield instances
    finally:
        for instance in instances:
            instance.cleanup()


class Plugin(object):
    """
    Interface definition for plugins that can interact with the docker-compose
    process.
    """

    def __init__(self, env, name, command):
        self.name = name
        self.env = env.copy()
        self.log = logging.getLogger(__name__)

    def initialize(self):
        return True

    def cleanup(self):
        pass

    def environment(self):
        return {}

    @property
    def basedir(self):
        return self.env['DORK_SOURCE']

    @property
    def datadir(self):
        return os.path.expanduser(self.env['DORK_DATA_DIR'])

    @property
    def lockdir(self):
        return '%s/locks' % self.datadir

    @property
    def project(self):
        return self.env['DORK_PROJECT']

    @property
    def instance(self):
        return self.env['DORK_INSTANCE']

    def info(self, project):
        return {}

    def alter_config_schema(self, schema):
        pass

    def preprocess_config(self, config):
        """
        Alter the docker-compose configuration object. The object is passed
        by reference.

        :type config: compose.config.config.Config
        """
        pass

    def building(self, service, no_cache, pull, force_rm):
        pass

    def after_build(self, service, no_cache, pull, force_rm):
        pass

    def initializing(self, project, service_names=None):
        pass

    def creating_container(self, service):
        pass

    def starting_container(self, container):
        pass

    def initialized(self, project, containers=None):
        pass

    def removing(self, project, include_volumes=False):
        pass

    def removed(self, project, include_volumes=False):
        pass

    def snapshot_save(self, snapshots=(), volumes=()):
        """
        Save the current volumes under the names provided.
        :type snapshots: list[str]
        """
        pass

    def snapshot_load(self, snapshots=(), volumes=()):
        """
        Try to load the snapshots provided. If multiple snapshots are requested
        the last valid one should be used.
        Returns the id of the snapshot loaded.

        :type snapshots: list[str]
        :rtype: str
        """
        pass

    def snapshot_rm(self, snapshots=()):
        """
        Remove the list of snapshots. Invalid snapshots are silently ignored.
        Return a list of snapshot id's that actually have been removed.

        :type snapshots: list[str]
        :rtype: list[str]
        """
        return []

    def snapshot_ls(self):
        """
        List available snapshots. I the snapshots parameter is provided, reduce
        check for existence of these and return the narrowed list.

        :rtype: list[str]
        """
        return []

    def snapshot_autosave(self):
        """
        Choose an automatic name for the next snapshot.

        :rtype: str
        """
        return None

    def snapshot_autoload(self, snapshots=()):
        """
        Choose the most appropriate snapshot to be loaded from the list provided.
        If no snapshot applies, return [None].

        :type snapshots: list[str]
        :rtype: str
        """
        return None

    def snapshot_autoclean(self, snapshots=()):
        """
        Decide which snapshots in the list can be cleaned up safely, without
        loosing information.

        :type snapshots: list[str]
        :rtype: list[str]
        """
        return []

    @property
    def auxiliary_project(self):
        return None

    @property
    def auxiliary_project_name(self):
        return 'dork_aux_%s' % self.name

    def attach_auxiliary_project(self, network):
        if not self.auxiliary_project:
            return

        aux = self.get_auxiliary_project()

        if not os.path.exists(self.lockdir):
            os.makedirs(self.lockdir)
        lock = filelock.FileLock("%s/%s" % (self.lockdir, self.auxiliary_project_name))

        with lock.acquire(60):
            aux.up(detached=True, remove_orphans=True)

            client = docker_client(self.environment())

            containers = client.containers(filters={
                'label': [
                    'org.iamdork.auxiliary',
                    'com.docker.compose.project=%s' % self.auxiliary_project_name
                ],
            })

            for container in containers:
                if network not in container['NetworkSettings']['Networks']:
                    client.connect_container_to_network(container, network)

    def detach_auxiliary_project(self, network):
        if not self.auxiliary_project:
            return

        aux = self.get_auxiliary_project()

        if not os.path.exists(self.lockdir):
            os.makedirs(self.lockdir)
        lock = filelock.FileLock("%s/%s" % (self.lockdir, self.auxiliary_project_name))

        with lock.acquire(60):

            client = docker_client(self.environment())

            containers = client.containers(filters={
                'label': [
                    'org.iamdork.auxiliary',
                    'com.docker.compose.project=%s' % self.auxiliary_project_name
                ],
            })

            for container in containers:
                if network in container['NetworkSettings']['Networks']:
                    if (len(container['NetworkSettings']['Networks']) - 1) == len(aux.networks.networks):
                        aux.down(remove_image_type=None, include_volumes=False, remove_orphans=True)
                        break
                    else:
                        client.disconnect_container_from_network(container, network)

    def get_auxiliary_project(self):
        config_details = config.find(self.auxiliary_project, [], Environment(self.environment()))
        project_name = self.auxiliary_project_name
        config_data = dork_config_load([], config_details)

        client = get_client(self.environment(), version=API_VERSIONS[config_data.version])

        return Project.from_config(project_name, config_data, client)



import dork_compose.plugin
import pkg_resources
import hvac
import os


class Plugin(dork_compose.plugin.Plugin):

    def __init__(self, env, name, command):
        dork_compose.plugin.Plugin.__init__(self, env, name, command)
        self.tokens = []

    def environment(self):
        return {
            'VAULT_VERSION': self.vault_version,
            'VAULT_ROOT_TOKEN': self.vault_root_token,
            'VAULT_HOST': self.vault_host,
            'VAULT_BUILD_HOST': self.vault_build_host,
            'VAULT_PORT': self.vault_port,
            'VAULT_MODE': self.vault_mode,
        }

    @property
    def vault_mode(self):
        return self.env.get('VAULT_MODE', 'secure')

    @property
    def vault_version(self):
        return self.env.get('VAULT_VERSION', '0.6.1')

    @property
    def vault_root_token(self):
        return self.env.get('VAULT_ROOT_TOKEN', 'dork')

    @property
    def vault_host(self):
        return self.env.get('VAULT_HOST', '127.0.0.1')

    @property
    def vault_port(self):
        return self.env.get('VAULT_PORT', '8200')

    @property
    def vault_build_host(self):
        return self.env.get('VAULT_BUILD_HOST', '172.17.0.1')

    @property
    def vault_secrets(self):
        return self.env.get('VAULT_SECRETS', '').split(';')

    @property
    def auxiliary_project(self):
        return pkg_resources.resource_filename('dork_compose', 'auxiliary/vault')

    def building(self, service,  no_cache, pull, force_rm):
        if 'args' in service.options['build'] and 'VAULT_TOKEN' in service.options['build']['args']:
            client = hvac.Client(
                url="http://%s:%s" % (self.vault_host, self.vault_port),
                token=self.vault_root_token
            )

            if self.vault_mode == 'secure':
                token = client.create_token()['auth']['client_token']
                self.tokens.append(token)
            else:
                token = self.vault_root_token

            for secret in self.vault_secrets:
                value = self.env.get(secret)
                if not value:
                    raise ValueError("Environment variable %s required as secret is not defined." % secret)
                if os.path.isfile(os.path.expanduser(value)):
                    with open(os.path.expanduser(value), 'r') as f:
                        value = f.read()
                client.write('secret/%s' % secret, value=value)
            service.options['build']['args']['VAULT_TOKEN'] = token
            service.options['build']['args']['VAULT_ADDR'] = 'http://%s:%s' % (
                self.vault_build_host, self.vault_port
            )

    def cleanup(self):
        client = hvac.Client(
            url="http://%s:%s" % (self.vault_host, self.vault_port),
            token=self.vault_root_token
        )
        for token in self.tokens:
            client.revoke_token(token)




import os
from functools import partial


import plugin

import compose.config.config
import compose.config.validation
import compose.project
import compose.cli.command
import compose.cli.main
import compose.cli.utils
from compose.cli.docopt_command import NoSuchCommand

from injections import DorkTopLevelCommand, get_dork_project, get_dork_project_name, dork_config_load, dork_validate_against_config_schema, dork_validate_service_constraints, dork_perform_command, dork_get_version_info
from compose.config.environment import env_vars_from_file

# Default plugins:
# lib:   Switch to dork library.
# multi: Assume single layer of multiple different projects.
# repo:  Turn on git auto snapshots.
# proxy: Run the proxy service.
DEFAULT_PLUGINS = 'env:multi:lib:autobuild:hotcode:filesystem:proxy:dns:vault'


def run():

    command = None

    # Replace compose TopLevelCommand with custom derivation with additional
    # commands.
    compose.cli.main.TopLevelCommand = DorkTopLevelCommand
    compose.cli.main.get_version_info = dork_get_version_info

    try:
        command = compose.cli.main.dispatch().args[1].__name__
    except NoSuchCommand:
        pass

    with plugin.load(os.getenv('DORK_PLUGINS', DEFAULT_PLUGINS), command) as plugins:

        compose.cli.main.perform_command = dork_perform_command
        # Override all occurences of config schema related functions.
        compose.config.config.validate_against_config_schema = partial(dork_validate_against_config_schema, plugins)
        compose.config.validate_service_constraints = partial(dork_validate_service_constraints, plugins)
        compose.config.config.validate_service_constraints = partial(dork_validate_service_constraints, plugins)

        # Inject configuration hooks.
        compose.config.config.load = partial(dork_config_load, plugins)

        # Inject custom get_dork_project to replace instances with DorkProjects.
        compose.cli.command.get_project = partial(get_dork_project, plugins)

        # Inject custom project_name function to allow dashes in project names.
        compose.cli.command.get_project_name = get_dork_project_name

        # Run the original dork-compose main function.
        compose.cli.main.main()







import dork_compose.plugin
from docker.api.client import APIClient


class Plugin(dork_compose.plugin.Plugin):

    def cleanup(self):
        client = APIClient()
        # Remove unused volumes.
        volumes = client.volumes({'dangling': True})
        if volumes and volumes['Volumes']:
            for volume in volumes['Volumes']:
                try:
                    client.remove_volume(volume['Name'])
                except Exception:
                    pass

        # Remove unused images.
        images = client.images(filters={'dangling': True})
        if images:
            for image in images:
                try:
                    client.remove_image(image['Id'], force=True)
                except Exception:
                    pass



import re

import os
from compose.cli.command import get_project
import compose.cli.command as cmd
from compose.cli.main import TopLevelCommand, perform_command
from compose.cli.utils import get_version_info
from compose.config.config import load
from compose.config.environment import Environment
from compose.const import DEFAULT_TIMEOUT
from compose.project import Project, ProjectNetworks
from compose.service import ConvergenceStrategy, BuildAction, Service
from compose.config.validation import load_jsonschema, get_resolver_path, \
    handle_errors, process_config_schema_errors, \
    process_service_constraint_errors
from dork_compose.helpers import tru

from functools import partial

from jsonschema import Draft4Validator
from jsonschema import FormatChecker
from jsonschema import RefResolver
import dork_compose


import logging
log = logging.getLogger(__name__)


def dork_get_version_info(scope):
    return '%s%sdork-compose version %s, build %s ' % (
        get_version_info(scope),
        ', ' if scope == 'compose' else '\n',
        dork_compose.__version__,
        dork_get_build_version()
    )


def dork_get_build_version():
    filename = os.path.join(os.path.dirname(dork_compose.__file__), 'GITSHA')
    if not os.path.exists(filename):
        return 'unknown'
    with open(filename) as fh:
        return fh.read().strip()


def dork_perform_command(options, handler, command_options):
    if '--timeout' in options and not options['--timeout']:
        options['--timeout'] = os.environ.get('DORK_DEFAULT_TIMEOUT', DEFAULT_TIMEOUT)
    return perform_command(options, handler, command_options)


def dork_config_load(plugins, config_details):
    config_data = load(config_details)
    for plugin in plugins:
        plugin.preprocess_config(config_data)
    return config_data


def get_dork_project(plugins, project_dir, config_path=None, project_name=None,
                     verbose=False, host=None, tls_config=None,
                     environment=None):

    cmd.config.load = partial(dork_config_load, plugins)
    project = get_project(project_dir, config_path, project_name, verbose, host, tls_config, environment)

    if 'COMPOSE_PROJECT_NAME' in os.environ:
        project.name = os.environ['COMPOSE_PROJECT_NAME']

    return DorkProject.from_project(project, plugins)


def get_dork_project_name(working_dir, project_name=None, environment=None):
    def normalize_name(name):
        # Full copy because compose strips dashes from project names.
        return re.sub(r'[^a-z0-9\-]', '', name.lower())

    if not environment:
        environment = Environment.from_env_file(working_dir)
    project_name = project_name or environment.get('COMPOSE_PROJECT_NAME')
    if project_name:
        return normalize_name(project_name)

    project = os.path.basename(os.path.abspath(working_dir))
    if project:
        return normalize_name(project)

    return 'default'


def dork_validate_service_constraints(plugins, config, service_name, version):
    def handler(errors):
        return process_service_constraint_errors(errors, service_name, version)
    schema = load_jsonschema(version)

    for plugin in plugins:
        plugin.alter_config_schema(schema)

    validator = Draft4Validator(schema['definitions']['constraints']['service'])
    handle_errors(validator.iter_errors(config), handler, None)


def dork_validate_against_config_schema(plugins, config_file):
    schema = load_jsonschema(config_file.version)
    for plugin in plugins:
        plugin.alter_config_schema(schema)
    format_checker = FormatChecker(["ports", "expose"])
    validator = Draft4Validator(
        schema,
        # TODO: wait for fix in docker-compose
        # docker-compose does not append filename, therefore caches
        # always miss.
        resolver=RefResolver(get_resolver_path() + "config_schema_v{0}.json".format(config_file.version), schema),
        format_checker=format_checker)
    handle_errors(
        validator.iter_errors(config_file.config),
        process_config_schema_errors,
        config_file.filename)


class DorkTopLevelCommand(TopLevelCommand):
    __doc__ = TopLevelCommand.__doc__ + "\n".join([
        "  snapshot           Save or restore runtime data snapshots.",
        "      info               Display information about services."
    ])

    def __init__(self, project, project_dir='.'):
        super(DorkTopLevelCommand, self).__init__(project, project_dir)

    def snapshot(self, options):
        """
        Save or restore volume snapshots.
        Usage: snapshot COMMAND [SNAPSHOTS...]

        Commands:
          save   Store volumes state as snapshot.
          load   Load the closest snapshot or a specific one.
          ls     List all available snapshots.
          rm     Clean up snapshots or remove a specific one.
        """
        getattr(self.project, 'snapshot_' + options['COMMAND'])(options['SNAPSHOTS'])

    def info(self, options):
        """
        Display service status information.

        Usage: info
        """

        from terminaltables import AsciiTable
        rows = []

        for key, value in self.project.info().iteritems():
            rows.append([key + ':', value])

        table = AsciiTable(rows)
        table.outer_border = False
        table.inner_column_border = False
        table.inner_heading_row_border = False
        table.title = 'Dork status information'
        print table.table


class Pluggable(object):

    def set_plugins(self, plugins):
        self.__plugins = plugins

    @property
    def plugins(self):
        return self.__plugins


class DorkService(Service, Pluggable):

    @classmethod
    def from_service(cls, service, plugins=()):
        service.__class__ = cls
        service.set_plugins(plugins)
        return service

    def build(self, no_cache=False, pull=False, force_rm=False):
        for plugin in self.plugins:
            plugin.building(self, no_cache, pull, force_rm)

        result = super(DorkService, self).build(no_cache, pull, force_rm)

        for plugin in self.plugins:
            plugin.after_build(self, no_cache, pull, force_rm)

        return result

    def start_container(self, container):
        for plugin in self.plugins:
            plugin.starting_container(container)
        return super(DorkService, self).start_container(container)

    def create_container(self, one_off=False, previous_container=None,
                         number=None, quiet=False, **override_options):

        for plugin in self.plugins:
            plugin.creating_container(self)
        return super(DorkService, self).create_container(one_off,
                                                         previous_container,
                                                         number, quiet,
                                                         **override_options)


class DorkNetworks(ProjectNetworks, Pluggable):
    def initialize(self):
        super(DorkNetworks, self).initialize()

        for key, network in self.networks.iteritems():
            for plugin in self.plugins:
                plugin.attach_auxiliary_project(network.full_name)

    def remove(self):
        for key, network in self.networks.iteritems():
            for plugin in self.plugins:
                plugin.detach_auxiliary_project(network.full_name)

        super(DorkNetworks, self).remove()


class DorkProject(Project, Pluggable):

    @classmethod
    def from_project(cls, project, plugins=()):
        project.__class__ = cls
        project.set_plugins(plugins)
        project.networks.__class__ = DorkNetworks
        project.networks.set_plugins(plugins)
        return project

    @classmethod
    def from_config(cls, name, config_data, client, plugins=()):
        project = super(DorkProject, cls).from_config(name, config_data, client)
        project.set_plugins(plugins)
        project.networks.set_plugins(plugins)
        return project

    def get_service(self, name):
        return DorkService.from_service(super(DorkProject, self).get_service(name), self.plugins)

    def up(self, service_names=None, start_deps=True, strategy=ConvergenceStrategy.changed, do_build=BuildAction.none, timeout=DEFAULT_TIMEOUT, detached=False, remove_orphans=False):
        for plugin in self.plugins:
            plugin.initializing(self, service_names)

        containers = super(DorkProject, self).up(service_names, start_deps, strategy, do_build, timeout, detached, remove_orphans)

        for plugin in self.plugins:
            plugin.initialized(self, containers)

        return containers

    def down(self, remove_image_type, include_volumes, remove_orphans=False):
        for plugin in self.plugins:
            plugin.removing(self, include_volumes)

        super(DorkProject, self).down(remove_image_type, include_volumes, remove_orphans)

        for plugin in self.plugins:
            plugin.removed(self, include_volumes)

    def __snapshots(self):
        snapshots = []
        for plugin in self.plugins:
            snapshots.extend(plugin.snapshot_ls())
        return snapshots

    def snapshot_save(self, names=()):
        # If the provided names list is empty, collect plugins for
        # name suggestions from autosave plugins.
        if not names:
            names = filter(tru, [p.snapshot_autosave() for p in self.plugins])

        # Invoke plugin save hooks with collected names.
        self.stop()
        for plugin in self.plugins:
            plugin.snapshot_save(names, self.volumes.volumes)
        self.start()

    def snapshot_load(self, names=()):
        # If the names list is empty, collect most appropriate snapshots
        # from autoload plugins.
        if not names:
            names = filter(tru, [p.snapshot_autoload(self.__snapshots()) for p in self.plugins])

        # Iterate plugins from the right and stop when the first one
        # successfully loaded a snapshot.

        self.stop()
        for plugin in reversed(self.plugins):
            loaded = plugin.snapshot_load(names, self.volumes.volumes)
            if loaded:
                log.info("Loaded snapshot %s through plugin %s." % (loaded, plugin.name))
                break
        self.start()

    def snapshot_rm(self, names=()):
        # If the names list is empty, collect most removable snapshots
        # from autoclean plugins.
        if not names:
            names = filter(tru, [p.snapshot_autoclean(self.__snapshots()) for p in self.plugins])

        for plugin in self.plugins:
            for removed in plugin.snapshot_rm(names):
                log.info("Removed snapshot %s through plugin %s." % (removed, plugin.name))

    def snapshot_ls(self, snapshots=()):
        for name in self.__snapshots():
            if not snapshots or name in snapshots:
                log.info(name)

    def info(self):
        info = {}
        for plugin in self.plugins:
            info.update(plugin.info(self))
        return info





"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

import dork_compose

# Always prefer setuptools over distutils
from setuptools import setup, find_packages


setup(
    name='dork-compose',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=dork_compose.__version__,

    description='docker-compose with magicks.',

    # The project's main homepage.
    url='https://github.com/iamdork/dork',

    # Author details
    author='Philipp Melab',
    author_email='philipp.melab@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        # 3 - Alpha
        # 4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],

    # What does your project relate to?
    keywords='docker docker-compose devops development',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    package_data={'dork_compose': [
        'plugins/*.py',
        'auxiliary/*/docker-compose.yml',
        'auxiliary/*/docker-compose.yml',
        'auxiliary/proxy/*/*',
    ]},

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'docker-compose==1.10.1',
        'gitpython',
        'terminaltables',
        'filelock',
        'hvac'
    ],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['check-manifest', 'minimock'],
    },

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'dork-compose=dork_compose.main:run',
        ],
    },
)


import dork_compose.plugin
from compose.config.config import VolumeSpec
from docker.api.client import APIClient
import logging
log = logging.getLogger(__name__)
import time

from docker.errors import APIError


class Plugin(dork_compose.plugin.Plugin):

    def get_hotcode_volumes(self, service):
        root = None
        source = '.'
        hotcode = ''
        try:
            image = service.client.inspect_image(service.image_name)
            if image.get('Config', {}).get('Labels'):
                root = image.get('Config', {}).get('Labels', {}).get('dork.root')
                source = image.get('Config', {}).get('Labels', {}).get('dork.source', '.')
                hotcode = image.get('Config', {}).get('Labels', {}).get('dork.hotcode', '')
        except APIError:
            pass

        if isinstance(service.options.get('labels'), dict):
            root = service.options.get('labels', {}).get('dork.root', root)
            source = service.options.get('labels', {}).get('dork.source', source)
            hotcode = service.options.get('labels').get('dork.hotcode', hotcode)

        if hotcode and not root:
            log.warn('Service %s has no dork.root information, can\'t apply hot code directories.' % service.name)
            return []

        paths = filter(lambda x: x, hotcode.split(';'))

        return [VolumeSpec.parse(':'.join([
            '%s/%s/%s' % (self.env['DORK_SOURCE'], source, path),
            '%s/%s' % (root.rstrip('/'), path),
            'rw'
        ])) for path in paths]

    def creating_container(self, service):
        """
        Inject volumes for all hot code paths.
        """
        self.sync_code(service=service)
        externals = [v.external for v in service.options['volumes']]
        for v in self.get_hotcode_volumes(service):
            if v.external not in externals:
                service.options['volumes'].append(v)

    def sync_code(self, service):
        client = APIClient()
        root = None
        source = '.'
        hotcode = ''
        try:
            image = service.client.inspect_image(service.image_name)
            if image.get('Config', {}).get('Labels'):
                root = image.get('Config', {}).get('Labels', {}).get('dork.root')
                source = image.get('Config', {}).get('Labels', {}).get('dork.source', '.')
                hotcode = image.get('Config', {}).get('Labels', {}).get('dork.hotcode', '')
        except APIError:
            pass

        if isinstance(service.options.get('labels'), dict):
            root = service.options.get('labels', {}).get('dork.root', root)
            source = service.options.get('labels', {}).get('dork.source', source)
            hotcode = service.options.get('labels').get('dork.hotcode', hotcode)

        skip = filter(lambda x: x, hotcode.split(';')) if hotcode else []

        # Don't sync if there are no hotcode directories.
        if not skip or '.' in skip:
            return

        skip.append('.git')
        skip.append('.env')
        skip.append('.dork.env')

        if not (source and root):
            return

        try:
            client.inspect_image('iamdork/rsync')
        except APIError:
            client.pull('iamdork/rsync')

        container = client.create_container(
            image=service.image_name,
            volumes=[root],
        )['Id']

        try:
            dork_source = self.env.get('DORK_SOURCE')
            src = root
            dst = '/'.join([dork_source, source])
            log.info("Synching %s to %s." % (src, dst))
            sync = client.create_container(
                image='iamdork/rsync',
                volumes=['/destination'],
                cpu_shares=256,
                host_config=client.create_host_config(
                    binds=['%s:/destination' % dst],
                    volumes_from=container
                ),
                environment={
                    'SOURCE': src,
                    'EXCLUDE': ' '.join(skip)
                }
            )['Id']
            try:
                client.start(sync)
                while client.inspect_container(sync)['State']['Running']:
                    time.sleep(0.5)
            finally:
                client.remove_container(sync)
        finally:
            client.remove_container(container)





